package net.pricefx.common.api;

import java.util.Arrays;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

public enum WorkflowType {
	QUOTE("quote"),
	PRICEGRIDITEM("pricegriditem"),
	REBATEAGREEMENT("rebateagreement"),
	CONTRACT("contract"),
	DATACHANGEREQUEST("dcr"),
	PRICELIST("pricelist"),
	REBATERECORD("rebaterecord"),
	MODELRECORD("modelrecord"),
	CLAIM("claim"),
	DEALPLAN("dealplan"),
	MODEL("model"),
	CUSTOMFORM("customform"),
	COMPENSATION("compensation"),
	COMPENSATIONRECORD("compensationrecord");

	private final String typeName;

	WorkflowType(String id) {
		this.typeName = id;
	}

	private static final Map<String, WorkflowType> lookupViaName = Arrays.stream(values()).collect(Collectors.toMap(WorkflowType::typeName, Function.identity()));

	@Override
	public String toString() {
		return typeName();
	}

	public String typeName() {
		return typeName;
	}

	public static WorkflowType fromName(String name) {
		return Optional.ofNullable(nullableFromName(name))
				.orElseThrow(() -> new IllegalArgumentException("Cannot resolve " + name + " in " + WorkflowType.class.getName()));
	}

	public static WorkflowType nullableFromName(String dbValue) {
		return lookupViaName.get(dbValue);
	}
}
