package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.SortType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class HistogramChart extends AbstractBuilder {

	public HistogramChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private HistogramOptions options;
	private HistogramSeries series;
	private Dictionary<HistogramChart> dictionary;
	
	public HistogramSeries getSeries() {
		String id = addInvocation("getSeries");
		if (series == null) 
			series = new HistogramSeries(this, id, "Series", idGenerator, invocations);
		return series;
	}
	
	public HistogramOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new HistogramOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<HistogramChart> addAuxLine() {
		
		String id = addInvocation("addAuxLine");
		return new AuxLine<HistogramChart>(this, id, "AuxLine");
	}
	
	public Dictionary<HistogramChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.HISTOGRAM;
		return r;
	}
	
	public static class HistogramSeries extends AbstractSeries<HistogramChart,HistogramSeries> {

		public HistogramSeries(HistogramChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
		public HistogramSeries setHideDataLabels(Boolean hideDataLabels) {
			
			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		

		public AggrFieldAxis<HistogramSeries> setAxisX(String axisX) {
			
			String id = addInvocation("setAxisX",axisX);
			return new AggrFieldAxis<HistogramSeries>(this, id, "AxisX");
		}
		
		public HistogramSeries setBins(BigDecimal bins) {
			
			addInvocation("setBins",bins);
			return this;
		}
		
		public HistogramSeries setAxisY(String axisY) {
			
			addInvocation("setAxisY",axisY);
			return this;
		}
		
		public HistogramSeries withSortByAxisX(SortType sortType) {
			addInvocation("withSortByAxisX",sortType);
			return this;
		}

		public HistogramSeries withSortByAxisY(SortType sortType) {			
			addInvocation("withSortByAxisY",sortType);
			return this;
		}

		public Formula<HistogramSeries> setFormula() {
			String id = addInvocation("setFormula");
			return new Formula<HistogramSeries>(this, id, "Formula");
		}

		public HistogramAdditionalMeasure<HistogramSeries> addAdditionalMeasure() {

		    String id = addInvocation("addAdditionalMeasure");
		    return new HistogramAdditionalMeasure<HistogramSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}

		public HistogramSeries setGeneratedQueryDto(String generatedQueryDto) {
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}
	}
	
	public static class HistogramOptions extends AbstractOptions<HistogramChart,HistogramOptions> {

		public HistogramOptions(HistogramChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
	}
	
    public static class HistogramAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,HistogramAdditionalMeasure<ATTACHEDTO>> {
        
        public HistogramAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
            super(instanceId, classId, attachedTo, idGenerator, invocations);
        }
        
        public Expression<HistogramAdditionalMeasure<ATTACHEDTO>> withExpression() { 

            String id = addInvocation("withExpression");
            return new Expression<HistogramAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
        }       
    }	
	
}