package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.domain.LookupTable;
import net.pricefx.server.dto.calculation.ContextParameter;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Builder class for input type Lookup {@link InputType#LOOKUP}.
 */
public class VLookupBuilder extends AbstractInputBuilder<VLookupBuilder, Object> {

    private List<String> attributeNames;
    private Map<String, Object> keys;

    protected VLookupBuilder(InputBuilderContext builderContext, String paramName) {
        super(builderContext);

        LookupTable lt = null;

        cp = new ContextParameter(lt, builderContext.getEngineContext().getTargetDate());

    }

    /**
     * Set new attribute for the VLookupBuilder instance.
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createVLookup("lookup")
     *      .setAttributeName("myFavoriteLookup")
     *      .setKeys([KEY1: "key1", KEY2: "key2"])
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param attributeName name of the attribute, as a String
     * @return self
     */
    public VLookupBuilder setAttributeName(String attributeName) {
        if (attributeNames == null)
            attributeNames = new ArrayList<String>();
        attributeNames.add(attributeName);

        return this;
    }

    /**
     * Set new list of the attributes for the VLookupBuilder instance.
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createVLookup("lookup")
     *      .setAttributeNames(["name1", "name2"])
     *      .setKeys([KEY1: "key1", KEY2: "key2"])
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param attributeNames names of the attributes, as a List
     * @return self
     */
    public VLookupBuilder setAttributeNames(List<String> attributeNames) {
        this.attributeNames = attributeNames;
        return this;
    }

    /**
     * Set Map of keys for the VLookupBuilder instance.
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createVLookup("lookup")
     *      .setAttributeNames(["name1", "name2"])
     *      .setKeys([KEY1: "key1", KEY2: "key2"])
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param keys keys to be set, as a Map
     * @return self
     */
    public VLookupBuilder setKeys(Map keys) {
        this.keys = keys;
        return this;
    }

}
