package net.pricefx.logic.core.model.holders;

import net.pricefx.logic.core.model.BaseMapNullable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static net.pricefx.common.api.PriceFxInterface.FIELD_UNIQUENAME;

public class WorkflowStepHolder extends BaseMapNullable {
    private final static String STEP_NAME = "Step";
    private final static String STEP_APPROVER_GROUP = "Approvers";
    private final static String STEP_EXECUTION_STATUS = "Execution status";
    private final static String STEP_LAST_ACTION_TIME = "Last Action Time";
    private final static String STEP_REASON = "Reason";

    private List<WorkflowStepHolder> subSteps;

    public WorkflowStepHolder(Map<String, Object> data) {
        put(STEP_NAME, data.get(FIELD_UNIQUENAME));
        put(STEP_APPROVER_GROUP, data.get("approver"));
        put(STEP_EXECUTION_STATUS, data.get("executionStatus"));
        put(STEP_REASON, data.get("reason"));
        put(STEP_LAST_ACTION_TIME, data.get("lastAccess"));

        List<Map<String, Object>> subSteps = (List<Map<String, Object>>) data.get("subSteps");
        this.subSteps = subSteps == null ? new ArrayList<>() : subSteps
                .stream()
                .map(step -> new WorkflowStepHolder(step))
                .collect(Collectors.toList());
    }

    public static List<String> getAllColumns() {
        return Arrays.asList(STEP_NAME, STEP_APPROVER_GROUP, STEP_EXECUTION_STATUS, STEP_LAST_ACTION_TIME, STEP_REASON);
    }

    public List<WorkflowStepHolder> getSubSteps() {
        return subSteps;
    }
}
